/***************************************************************************
 *   Copyright (C) 2004 by bysk                                           *
 *   buysk@pf.pl                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "led_notify.h"

#include <X11/Xlib.h>

LedNotify* led_notify_session;

extern "C" int led_notify_init()
{
	kdebugf();
	led_notify_session = new LedNotify();
	kdebugf2();
	return 0;
}

extern "C" void led_notify_close()
{
	kdebugf();
	delete led_notify_session;
	led_notify_session = NULL;
	kdebugf2();
}

LedNotify::LedNotify(QObject* parent, const char* name)
{
	blinking = false;

	config_file.addVariable("Notify", "NewChat_LED", false);
	config_file.addVariable("Notify", "NewMessage_LED", false);
	
	ConfigDialog::addVGroupBox("Notify", "Notify", QT_TRANSLATE_NOOP("@default", "LED configuration"));
	ConfigDialog::addSpinBox("Notify", "LED configuration", QT_TRANSLATE_NOOP("@default", "Blinking delay (msec):"),"LEDdelay",50,10000,1,500);
	
	QMap<QString, QString> s;
	s["NewChat"] = SLOT(newChat(const UinsList &, const QString &, time_t));
	s["NewMessage"] = SLOT(newMessage(const UinsList &, const QString &, time_t, bool &));
	
	notify->registerNotifier("LED", this, s);
	
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(blink()));
	
	//theDisplay = XOpenDisplay(NULL);
}

LedNotify::~LedNotify()
{
	disconnect(timer, SIGNAL(timeout()), this, SLOT(blink()));
	notify->unregisterNotifier("LED");
	
	ConfigDialog::removeControl("Notify", "Blinking delay (msec):");
	ConfigDialog::removeControl("Notify", "LED configuration");
	
	//XCloseDisplay(theDisplay);
}

void LedNotify::newChat(const UinsList &senders, const QString& msg, time_t time)
{
	kdebugf();
	
	if(blinking)
		return;
	
	msgOrChat = false;
	blinking = true;
	led_status = false;
	
	timer->start(config_file.readNumEntry("Notify", "LEDdelay"));
	blink();
	kdebugf2();
}

void LedNotify::newMessage(const UinsList &senders, const QString& msg, time_t time, bool &grab)
{
	kdebugf();
	
	if(blinking)
		return;
	
	newMsgSenders = senders;
	Chat* chat = chat_manager->findChatByUins(newMsgSenders);
	if(chat->isActiveWindow())
		return;
	
	msgOrChat = true;
	blinking = true;
	led_status = false;
	
	timer->start(config_file.readNumEntry("Notify", "LEDdelay"));
	blink();
	kdebugf2();
}

void LedNotify::blink(void)
{
	kdebugf();

	static Display* theDisplay = QPaintDevice::x11AppDisplay();
	static bool bilet_wstepu;
	
	bilet_wstepu = true;
	
	if(blinking)
	{
		if(msgOrChat)
		{
			Chat* chat = chat_manager->findChatByUins(newMsgSenders);
			if(chat != NULL)
			{
				if(chat->isActiveWindow())
					bilet_wstepu = false;
			}
			else
				bilet_wstepu = false;
		}
		else
		{
			if(!pending.pendingMsgs())
				bilet_wstepu = false;
		}
	}
	else
		bilet_wstepu = false;
		
	if(!bilet_wstepu)
	{
		timer->stop();
		blinking = false;
		if(!led_status)
			return;
	}
	
	static XKeyboardControl values;
	values.led = 3;
	
	if(led_status)
	{
		values.led_mode = LedModeOff;
		led_status = false;
	}
	else
	{
		values.led_mode = LedModeOn;
		led_status = true;
	}

	XChangeKeyboardControl(theDisplay, KBLedMode, &values);
	//XSync(theDisplay,False);
	kdebugf2();
}
